//==============================================================================
//	File:		EMAC150Transceiver.v
//	Author:		Chris Fletcher (http://cwfletcher.net)
//	Copyright:	Copyright 2010 UC Berkeley
//==============================================================================

//==============================================================================
//	Section:	License
//==============================================================================
//	Copyright (c) 2010, Regents of the University of California
//	All rights reserved.
//
//	Redistribution and use in source and binary forms, with or without modification,
//	are permitted provided that the following conditions are met:
//
//		- Redistributions of source code must retain the above copyright notice,
//			this list of conditions and the following disclaimer.
//		- Redistributions in binary form must reproduce the above copyright
//			notice, this list of conditions and the following disclaimer
//			in the documentation and/or other materials provided with the
//			distribution.
//		- Neither the name of the University of California, Berkeley nor the
//			names of its contributors may be used to endorse or promote
//			products derived from this software without specific prior
//			written permission.
//
//	THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
//	ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
//	WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
//	DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
//	ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
//	(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
//	LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
//	ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
//	(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
//	SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//==============================================================================

//------------------------------------------------------------------------------
//	Module:		EMAC150Transceiver
//	Desc:		An RX/TX EMAC wrapper.  You will add the missing logic 
//				(buffering and header assembly/filtering) in this module.  All 
//				of the EMAC-bound signals that you will have to interface with 
//				are described alongside the EMACPHY instantiation (below).  
//				Inside EMACPHY, you wil find:
//					1.) EMACWrapper
//					2.) mii_byte_phy_if
//				(1) innstantiates the actual EMAC primitive.  You might want to 
//				look at this if you plan to optimize your design by playing with 
//				the primitive itself.  You need not change anything in (2) -- it 
//				is only used to connect external clocks and resets.
//
//				A note on interfaces: The interface specification given for this 
//				module is **recommended only**.  You can change it as you see 
//				fit. 
//
//				DO NOT change any signals under "MII Interface."  These are 
//				Ethernet protocol-related and set in stone.
//	Author:		<a href="http://cwfletcher.net">Chris Fletcher</a>
//------------------------------------------------------------------------------
module EMAC150Transceiver(
			//------------------------------------------------------------------
			//	Clock & Reset Inputs
			//------------------------------------------------------------------
			Clock,
			Reset,
			//------------------------------------------------------------------
			
			//------------------------------------------------------------------
			//	MAC Configuration
			//------------------------------------------------------------------
			DstMAC, 
			SrcMAC,
			EthProt,
			//------------------------------------------------------------------
			
			//------------------------------------------------------------------
			//	RX Ports
			//------------------------------------------------------------------
			RXClock,
			RXReset,
			RXData,
			RXValid,
			RXReady,
			//------------------------------------------------------------------
			
			//------------------------------------------------------------------
			//	TX Ports
			//------------------------------------------------------------------
			TXClock,
			TXReset,
			TXData,
			TXFrameStop,
			TXValid,
			TXReady,
			//------------------------------------------------------------------
			
			//------------------------------------------------------------------
			//	MII Interface
			//------------------------------------------------------------------
			PHY_COL,
			PHY_CRS,
			PHY_TXD,
			PHY_TXCTL_TXEN,
			PHY_TXER,
			PHY_TXCLK,
			PHY_RXD,
			PHY_RXCTL_RXDV,
			PHY_RXER,
			PHY_RXCLK,
			//------------------------------------------------------------------
		);
	//--------------------------------------------------------------------------
	//	Parameters
	//--------------------------------------------------------------------------
	parameter				Width =					8;
	//--------------------------------------------------------------------------
	
	//--------------------------------------------------------------------------
	//	Constants
	//--------------------------------------------------------------------------
	localparam				WWidth =				Width + 1,
							EthAWidth =				48,
							EthProtWidth =			16;
	//--------------------------------------------------------------------------	

	//--------------------------------------------------------------------------
	//	Clock & Reset Inputs
	//--------------------------------------------------------------------------
    input           		Clock;
    input          			Reset;	
	//--------------------------------------------------------------------------
	
	//--------------------------------------------------------------------------
	//	MAC Configuration
	//--------------------------------------------------------------------------
	input	[EthAWidth-1:0]	DstMAC, SrcMAC;
	input	[EthProtWidth-1:0] EthProt;
	//--------------------------------------------------------------------------
    
    //--------------------------------------------------------------------------
	//	RX Ports
	//--------------------------------------------------------------------------
    input           		RXClock;
    input           		RXReset;
    output   [WWidth-1:0]  	RXData;
    output          		RXValid;
    input           		RXReady;
	//--------------------------------------------------------------------------

	//--------------------------------------------------------------------------
	//	TX Ports
	//--------------------------------------------------------------------------
    input           		TXClock;
    input           		TXReset;
    input    [Width-1:0]  	TXData;
    input           		TXFrameStop;
    input           		TXValid;
    output          		TXReady;
	//--------------------------------------------------------------------------

	//--------------------------------------------------------------------------
	//	MII Interface
	//--------------------------------------------------------------------------
    input           		PHY_COL;
    input           		PHY_CRS;
    output   [3:0]  		PHY_TXD;
    output          		PHY_TXCTL_TXEN;
    output          		PHY_TXER;
    input           		PHY_TXCLK;
    input    [3:0]  		PHY_RXD;
    input           		PHY_RXCTL_RXDV;
    input           		PHY_RXER;
    input           		PHY_RXCLK;
	//--------------------------------------------------------------------------

	//--------------------------------------------------------------------------
	// Wire & Regs 
	//--------------------------------------------------------------------------
    wire            		TXClockEMAC;
    wire            		RXClockEMAC;

    wire     [Width-1:0]  	TXDataEMAC;
    wire            		TXValidEMAC;
    wire            		TXAckEMAC;

    wire     [Width-1:0]  	RXDataEMAC;
    wire            		RXValidEMAC;
    wire            		RXGoodEMAC;
    wire            		RXBadEMAC;

    //synthesis attribute keep of TXDataEMAC is "true";
    //synthesis attribute keep of TXValidEMAC is "true";
    //synthesis attribute keep of TXAckEMAC is "true";
    //synthesis attribute keep of RXDataEMAC is "true";
    //synthesis attribute keep of RXValidEMAC is "true";
	//--------------------------------------------------------------------------
	
    //--------------------------------------------------------------------------
    //	EMAC Wrapper
    //--------------------------------------------------------------------------
    EMACPHY 		EMAC(		.TX_CLK_0(			Clock),
								.RESET(				Reset),
								
								.EMAC0CLIENTRXD(	RXDataEMAC),				/* The incoming data */
								.EMAC0CLIENTRXDVLD(	RXValidEMAC),				/* Is the incoming data valid? */
								.EMAC0CLIENTRXGOODFRAME(RXGoodEMAC),			/* Did the incoming packet have a compliant CRC? */
								.EMAC0CLIENTRXBADFRAME(RXBadEMAC),				/* Did it NOT have a compliant CRC? */
								.EMAC0CLIENTRXFRAMEDROP(/* Not Used */),
								.EMAC0CLIENTRXSTATS(/* Not Used */),
								.EMAC0CLIENTRXSTATSVLD(/* Not Used */),
								.EMAC0CLIENTRXSTATSBYTEVLD(/* Not Used */),

								.CLIENTEMAC0TXD(	TXDataEMAC),				/* The outgoing data */
								.CLIENTEMAC0TXDVLD(	TXValidEMAC),				/* Is the outgoing data valid? */
								.EMAC0CLIENTTXACK(	TXAckEMAC),					/* A pulse acknowledgement that the EMAC is now sending a packet's worth of data */
								.CLIENTEMAC0TXFIRSTBYTE(1'b0),					/* (The outgoing packet will be sent continuously during after after this signal is pulsed)*/
								.CLIENTEMAC0TXUNDERRUN(1'b0),					/* You should always send continuous packets, so this can be tied to 1'b0 */
								.EMAC0CLIENTTXCOLLISION(/* Not Used */),
								.EMAC0CLIENTTXRETRANSMIT(/* Not Used */),
								.CLIENTEMAC0TXIFGDELAY(8'b0),
								.EMAC0CLIENTTXSTATS(/* Not Used */),
								.EMAC0CLIENTTXSTATSVLD(/* Not Used */),
								.EMAC0CLIENTTXSTATSBYTEVLD(/* Not Used */),

								.CLIENTEMAC0PAUSEREQ(1'b0),
								.CLIENTEMAC0PAUSEVAL(16'b0),
								
								.MII_COL_0(			PHY_COL),
								.MII_CRS_0(			PHY_CRS),
								.MII_TXD_0(			PHY_TXD),
								.MII_TX_EN_0(		PHY_TXCTL_TXEN),
								.MII_TX_ER_0(		PHY_TXER),
								.MII_TX_CLK_0(		PHY_TXCLK),
								.MII_RXD_0(			PHY_RXD),
								.MII_RX_DV_0(		PHY_RXCTL_RXDV),
								.MII_RX_ER_0(		PHY_RXER),
								.MII_RX_CLK_0(		PHY_RXCLK));							
	//--------------------------------------------------------------------------

	//---------------------------------------------------------------------------------------------
	//---------------------------------------------------------------------------------------------
	//											Checkpoint 2
	//									(Only modify below this line)
	//---------------------------------------------------------------------------------------------
	//---------------------------------------------------------------------------------------------
					
    assign 	TXClockEMAC = 							Clock;					// Use this as the EMAC-side clock on the transmit side
    assign 	RXClockEMAC = 							PHY_RXCLK;				// Use this as the EMAC-side clock on the receive side
	
	/* You EMAC datapaths go here */
	
endmodule
//--------------------------------------------------------------------------